import os
import fileutils

android_task = '''curl \
    -F "status=2" \
    -F "notify=1" \
    -F "notes=None." \
    -F "notes_type=0" \
    -F "ipa=@{ipa}" \
    -H "X-HockeyAppToken: 520439d79df6430fb66b45937510cbad" \
    https://rink.hockeyapp.net/api/2/apps/{hockey_app_id}/app_versions/upload
'''

# ios_task = '''curl \
#     -F "status=2" \
#     -F "notify=1" \
#     -F "notes=None." \
#     -F "notes_type=0" \
#     -F "ipa=@ipa/{app}.ipa" \
#     -F "dsym=marines-mobile.app.dsym.zip" \
#     -H "X-HockeyAppToken: 520439d79df6430fb66b45937510cbad" \
#     https://rink.hockeyapp.net/api/2/apps/{hockey_app_id}/app_versions/upload
# '''


def hockey_app_upload(project, parser):
    if parser.platform == 'ios':
        print 'not supported hockey app upload for ios builds'
        exit(1)
    if not project.services[parser.platform].hockeyapp_id:
        exit(0)

    if not parser.ipa:
        parser.ipa = 'c:\\Build\\{project}\\outputs\\apk\\release\\{project}-release.apk'.format(
            root=fileutils.root_dir,
            project=project.project_name,
        )

    command = android_task
    command = command.replace('{ipa}', parser.ipa)
    command = command.replace('{hockey_app_id}', project.services[parser.platform].hockeyapp_id)
    print command

    result = False
    for i in xrange(5):
        result = 0 == os.system(command)
        if result:
            break

    if not result:
        print 'Cannot upload apk to hockey app'
        exit(2)



import parse_csv
import diagram

caption = ''


def remove_fakes(table):
    uids = table.select('uid')
    percent = 0.0
    for i, uid in enumerate(uids):
        if(uid == ''):
            continue
        count = table.count_if('uid', '"{}"=="%s"' % uid)
        if count > 2:
            table.remove_if('uid', '"{}"=="%s"' % uid)
        percent_ = round(float(i) / len(uids) * 100, 0)
        if percent_ > percent:
            percent = percent_
            print '{}%, rows={}'.format(percent, len(table.rows))


def purchases_by_level(table):
    count = table.max('level')
    values = []
    print 'Level:\tCount Purchases'
    for level in xrange(count + 1):
        value = table.count_if('level', '{}==%s' % level)
        print '{}\t{}'.format(level, value)
        values.append(value)
    diagram.build(values, caption, 'purchases_by_levels.png')


def main(argv):
    file = 'PurchaseSuccessful.csv'
    if len(argv) > 0:
        file = argv[0]

    global caption
    table = parse_csv.parse(file)
    remove_fakes(table)
    table.save('PurchaseSuccessful_raw.txt')
    caption = 'Purchases on levels\n' + file
    purchases_by_level(table)


if __name__ == '__main__':
    main([])



import xml.etree.ElementTree as ET
import game
import os
import fileutils
from project import Project
from validate_units import get_unit_type

logs = []


def log(msg):
    logs.append(msg)


def get_maps():
    result = []
    for i in xrange(100):
        file = fileutils.root_dir + '/Resources/ini/maps/map%d.xml' % i
        if os.path.isfile(file):
            result.append(file)
        for ii in xrange(100):
            file = fileutils.root_dir + '/Resources/ini/maps/map%d_%d.xml' % (i, ii)
            if os.path.isfile(file):
                result.append(file)
    return result


def check_unused_creeps(creeps):
    if not creeps:
        return

    units = game.get_units_path()
    files = []
    for unit, path in units.items():
        files.append(path)
    files.extend(get_maps())

    for file in files:
        data = open(file).read()

        index = 0
        while index < len(creeps):
            unit = creeps[index]
            position = data.find(unit)
            if position != -1:
                creeps.remove(unit)
            else:
                index += 1

        if not creeps:
            break

    for creep in creeps:
        print '  - Warning: detect unused creep [%s]'%creep


def validate_maps_content():
    units = game.get_units_list()
    unused_creeps = []
    for unit in units:
        if get_unit_type(unit) == 'creep':
            unused_creeps.append(unit)

    def use_unit(unit):
        if unit in unused_creeps:
            unused_creeps.remove(unit)

    files = get_maps()
    for file in files:
        root = ET.parse(file).getroot()

        waves = root.find('waves')
        if waves is None:
            waves = root.find('waves_hard')
        if waves is None:
            waves = root.find('waves_survival')
        if waves is None:
            waves = root.find('waves_tournament')
        if waves is None:
            log('Cannot find wave node in xml {}'.format(file))
            raise SystemError('Cannot find wave node in xml {}'.format(file))

        routes = root.find('routes')
        units_on_map = {}

        def validate_route(routeindex):
            try:
                int(routeindex)
            except:
                log('Route with index [{}] not is digit. {}'.format(routeindex, file))
                return False
            for route in routes:
                if route.attrib['name'] == routeindex:
                    return True
            log('Route with index [{}] not found in {}'.format(routeindex, file))
            return False

        def validate_routesubtype(rst):
            if rst not in ['main', 'left', 'right', 'random']:
                log('Unknow value of routesubtype [{}] in {}'.format(rst, file))

        for wave in waves:
            if 'defaultname' in wave.attrib:
                unit = wave.attrib['defaultname']
                if unit not in units:
                    log('Invalid name of unit [{}] in {}'.format(unit, file))
                units_on_map[unit] = 1
                use_unit(unit)
            if 'defaultrouteindex' in wave.attrib:
                validate_route(wave.attrib['defaultrouteindex'])
            if 'defaultroutesubtype' in wave.attrib:
                validate_routesubtype(wave.attrib['defaultroutesubtype'])
            for unitxml in wave:
                unit = unitxml.attrib['name'] if 'name' in unitxml.attrib else ''
                units_on_map[unit] = 1
                if unit and unit not in units:
                    log('Invalid name of unit [{}] in{}'.format(unit, file))
                if 'routeindex' in unitxml.attrib:
                    validate_route(unitxml.attrib['routeindex'])
                if 'routesubtype' in unitxml.attrib:
                    validate_routesubtype(unitxml.attrib['routesubtype'])
                use_unit(unit)
        if 'max_creeps_on_level' in Project.instance.validate:
            max_count = Project.instance.validate['max_creeps_on_level']
        else:
            max_count = 8
        if len(units_on_map) > max_count:
            log('Many creeps in {} ({}>{})'.format(file, len(units_on_map), max_count))

        for route in routes:
            main_points = len(route.find('main'))
            left_points = len(route.find('left'))
            right_points = len(route.find('right'))
            if main_points != left_points or main_points != right_points or left_points != right_points:
                log('Different count points in route. Map: {})'.format(file))
    check_unused_creeps(unused_creeps)


def validate_count():
    count_xml = 0
    count_rewards = 0
    count_locations = 0

    maps = get_maps()
    count_xml = len(maps)

    file = fileutils.root_dir + '/Resources/ini/maps/levels.xml'
    if not os.path.isfile(file):
        return
    rewards = ET.parse(fileutils.root_dir + '/Resources/ini/maps/levels.xml').getroot()
    for child in rewards:
        if child.tag.startswith('level_'):
            count_rewards += 1

    location_xml = ET.parse(fileutils.root_dir + '/Resources/ini/map/maplayer.xml')
    locations = location_xml.getroot().find('locations')
    if locations is None or len(locations) == 0:
        location_xml = ET.parse(fileutils.root_dir + '/Resources/ini/map/locations.xml')
        locations = location_xml.getroot().find('locations')
    for _ in locations:
        count_locations += 1
    for index in xrange(100):
        node = 'locations_%d' % index
        locations = location_xml.getroot().find(node)
        if locations is not None and len(locations):
            for _ in locations:
                count_locations += 1

    valid = count_locations == count_xml and count_locations == count_rewards
    if not valid:
        log('Count of levels is difference: \n\tlocations: {}\n\trewards: {}\n\txmls: {}'.
            format(count_locations, count_rewards, count_xml))


def validate():
    # try:
    validate_maps_content()
    validate_count()
    # except Exception as e:
    #     print(e)
    #     exit(-1)



import xml.etree.ElementTree as ET
from google_sheets import GoogleSheets
from project import Project
import fileutils
import validator_localization
from validate import print_log


class Locale:

    def __init__(self):
        self.values = ['', '']


def convert_to_table():
    locales = {}
    keys = []

    for i, lang in enumerate(['en', 'ru']):
        filepath = '{}/Resources/lang/{}.xml'.format(fileutils.root_dir, lang)
        root = ET.parse(filepath).getroot()
        dict_ = root.find('dict')
        key = ''
        value = ''
        for child in dict_:
            if child.tag == 'key':
                key = child.text
            elif child.tag == 'string':
                value = child.text
                if key and value and len(key) and len(value):
                    if key not in locales:
                        locale = Locale()
                        locales[key] = locale
                    locales[key].values[i] = value
                    # print len(locales), len(locales[key].values), locales[key], key, value

                    if key not in keys:
                        keys.append(key)
                    key = ''
                    value = ''

    for key in keys:
        print key + '\t' + '\t'.join(locales[key].values)


def download(document_id):

    languages = ['EN', 'RU', 'DE', 'FR', 'IT']

    gs = GoogleSheets(CLIENT_SECRET_FILE=Project.instance.gg_secret_file)
    gs.set_document(document_id)
    raw = gs.read_range(gs.get_sheet_titles()[0], 'A1', 'Z')
    header = raw[0]
    data = raw[1:]

    for lang in languages:
        if lang not in header:
            print '  - Skip [{}]'.format(lang)
            continue
        locales = []
        index_id = header.index('ID')
        index_lg = header.index(lang)
        for line in data:
            id = line[index_id]
            lg = line[index_lg]
            locale = Locale()
            locales.append([id, lg])

        lg = ''
        for item in locales:
            id = item[0]
            locale = item[1]
            if not id:
                continue
            lg += '        <key>{}</key><string>{}</string>\n'.format(id, locale.encode('utf-8'))
        lg = '<plist version="1.0">\n    <dict>\n' + lg + '    </dict>\n</plist>'
        fileutils.write('{}/Resources/lang/{}.xml'.format(fileutils.root_dir, lang.lower()), lg)
        print '  - Download [{}] finished'.format(lang)

    print_log(validator_localization, 'Validate localization:')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
import subprocess
import platform
from random import randint

import fileutils
from project import Project
from management import Slack


message_with_insufficient_test_quota = '''
Упс. Превышена квота на количество тестовых запусков в Firebase TestLab.
Придется проверять запуск apk вручную.
Проект: *{project_name}*
'''


def get_app_info(app_gradle_path):
    # data = open(app_gradle_path).read()
    # min_sdk_version = re.findall(r'minSdkVersion (\d+)', data)[0]
    # print('  - Min SDK Version: ', min_sdk_version)
    # return min_sdk_version

    # TODO
    return 26


def get_devices_list(app_info):
    arguments = [
        'gcloud',
        'firebase',
        'test',
        'android',
        'models',
        'list',
    ]
    process = subprocess.Popen(arguments, stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE,
                               shell=(platform.system() == "Windows"))
    out, err = process.communicate()
    print(out)
    # matches = re.findall(r'(\w+).+ \d+ x \d+.+\s([\d,]+)', out)
    matches = re.findall(r'(\w+)(.+)PHYSICAL.+ \d+ x \d+.+\s([\d,]+)', out)
    devices = []
    for match in matches:
        device_id = match[0]
        try:
            if 'watch' in match[1].lower():
                print('Skip %s as watch' % device_id)
                continue
            sdks = [int(x) for x in match[2].split(',')]
            sdks = list(filter(lambda x: x >= app_info, sdks))
        except ValueError:
            continue
        if sdks:
            devices.append((device_id, sdks))
    print devices
    return devices


def run_robo_tests(devices, path_to_apk):
    arguments = [
        'gcloud',
        'firebase',
        'test',
        'android',
        'run',
        '--type',
        'robo',
        '--app',
        path_to_apk,
    ]

    device_id = 'Any'
    os_sdk_id = 'Any'
    os_sdk_id

    if devices is not None:
        device = devices[randint(0, len(devices) - 1)]
        device_id = device[0]
        os_sdk_id = device[1][randint(0, len(device[1]) - 1)]
        os_sdk_id = str(os_sdk_id)

        arguments.extend([
            '--device-ids',
            device_id,
            '--os-version-ids',
            os_sdk_id,
        ])
    print('Run robo-test on device: %s OS: %s' % (device_id, os_sdk_id))

    process = subprocess.Popen(arguments, stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE,
                               shell=(platform.system() == "Windows"))
    out, err = process.communicate()
    print 'Return Code:', process.returncode
    print 'Out:\n', out
    print 'Err:\n', err

    if process.returncode == 1 and devices is not None:
        return run_robo_tests(None, path_to_apk)

    return process.returncode


def main(apk_path):
    app_gradle_path = '{}/proj.android/app/build.gradle'.format(fileutils.root_dir)

    app_info = get_app_info(app_gradle_path)
    devices = get_devices_list(app_info)
    result = run_robo_tests(devices, apk_path)
    if result == 1:
        message = message_with_insufficient_test_quota.format(project_name=Project.instance.app_bundle_name)
        Slack().send_message('#stores', message, 'Сборщик и тестировщик Android билдов')
        exit(0)
    exit(result)


if __name__ == '__main__':
    fileutils.root_dir = '/work/td_core/projects/syndicate-4'
    apk = '/users/stereo7/Downloads/syndicate4-release.apk'
    main(apk)



from __future__ import division
import os
import fileutils
import shutil
from project import Project
import string


def istext(filename):
    s = open(filename).read(512)
    text_characters = "".join(map(chr, range(32, 127)) + list("\n\r\t\b"))
    _null_trans = string.maketrans("", "")
    if not s:
        return True
    if "\0" in s:
        return False
    t = s.translate(_null_trans, text_characters)
    if float(len(t)) / float(len(s)) > 0.30:
        return False
    return True


def clone():
    out_folder = '{}/proj.android'.format(fileutils.root_dir)
    if os.path.isdir(out_folder):
        shutil.rmtree(out_folder)
    cmd = 'git clone git@bitbucket.org:stereo7_tools/template_proj_android_syn.git {}'.format(out_folder)
    os.system(cmd)
    shutil.rmtree('{}/.git'.format(out_folder))


def get_project_folder(arg_parser):
    folder = fileutils.root_dir + '/proj.android'
    if Project.instance.with_pro_version:
        folder = folder + '.' + arg_parser.app_kind
    folder += '/'
    return folder


def replace_values(arg_parser):
    folder = fileutils.root_dir + '/proj.android'
    if Project.instance.with_pro_version:
        out_folder = folder + '.' + arg_parser.app_kind
        if os.path.isdir(out_folder):
            shutil.rmtree(out_folder)
        os.rename(folder, out_folder)
        folder = out_folder
    folder += '/'

    project_name = Project.instance.project_name
    if Project.instance.with_pro_version:
        project_name += '.' + arg_parser.app_kind

    files = fileutils.getFilesList(folder)
    for file in files:
        file = folder + file
        if istext(file):
            content = open(file).read()
            package = Project.instance.package.replace('-', '')
            content = content.replace('@{PACKAGE_NAME}', package)
            content = content.replace('@{APP_BUNDLE_NAME}', Project.instance.app_bundle_name)
            content = content.replace('@{PROJECT_NAME}', project_name)
            if Project.instance.with_pro_version:
                kind = arg_parser.app_kind
                content = content.replace('@{FACEBOOK_APP_ID}', Project.instance.services['android'][kind].facebook_id)
                content = content.replace('@{FLURRY_APP_ID}', Project.instance.services['android'][kind].flurry_id)
                content = content.replace('@{GSM_APP_ID}', Project.instance.services['android'][kind].gsm_sender_id)
                content = content.replace('@{APPODEAL_APP_ID}', Project.instance.services['android'][kind].appodeal_id)
                content = content.replace('@{APP_KIND_DEFINE_FOLDER_ANDROID}', 'LOCAL_CPPFLAGS += -DRESOURCES_OVERRIDE_FOLDER=\\"%s\\"' % kind)
            else:
                content = content.replace('@{FACEBOOK_APP_ID}', Project.instance.services['android'].facebook_id)
                content = content.replace('@{FLURRY_APP_ID}', Project.instance.services['android'].flurry_id)
                content = content.replace('@{GSM_APP_ID}', Project.instance.services['android'].gsm_sender_id)
                content = content.replace('@{APPODEAL_APP_ID}', Project.instance.services['android'].appodeal_id)
                content = content.replace('@{APP_KIND_DEFINE_FOLDER_ANDROID}', '')

                content = content.replace('${ADMOB_APP_ID}', Project.instance.services['android'].admob_app_id)
                content = content.replace('${ADMOB_INTERSTITIAL}', Project.instance.services['android'].admob_interstitial)
                content = content.replace('${ADMOB_REWARDED}', Project.instance.services['android'].admob_rewarded)
                content = content.replace('${INAPP_KEY}', Project.instance.services['android'].inapp_key)
                content = content.replace('${AMPLITUDE_KEY}', Project.instance.services['android'].amplitude_key)
                content = content.replace('${TENJIN_KEY}', Project.instance.services['android'].tenjin_key)
                content = content.replace('${APPLOVIN_SDK_KEY}', Project.instance.services['android'].applovin_key)

            fileutils.write(file, content)


def copy_resources(arg_parser):
    proj_folder = get_project_folder(arg_parser) + 'app/res/'
    icons = '{}/store/icons/'.format(fileutils.root_dir)
    if Project.instance.with_pro_version:
        icons += arg_parser.app_kind + '/'
    shutil.copy2('{}drawable-ldpi/icon.png'.format(icons), '{}drawable-ldpi/icon.png'.format(proj_folder))
    shutil.copy2('{}drawable-mdpi/icon.png'.format(icons), '{}drawable-mdpi/icon.png'.format(proj_folder))
    shutil.copy2('{}drawable-hdpi/icon.png'.format(icons), '{}drawable-hdpi/icon.png'.format(proj_folder))
    shutil.copy2('{}drawable-xhdpi/icon.png'.format(icons), '{}drawable-xhdpi/icon.png'.format(proj_folder))


def run(arg_parser):
    Project.instance.check_custom_package('android')
    clone()
    replace_values(arg_parser)
    copy_resources(arg_parser)
